var App = (function($) {
	var app = {};
	
	/**
	 * 打开第三方分享页面
	 * @param {Object} data 需要分享的JSON信息 
	 */
	app.openShare = function(data) {
		url = '_www/html/share.html';
		$.openWindow(url, 'share', {
			show: {
				duration: 100,
				aniShow: 'none'
			},
			styles: {
				height: '40%',
				bottom: 0,
				top: '60%',
				background: "rgba(0,0,0,0.6)"
			},
			extras: {
				jsonData: data
			}
		});
	}
	
	
	/**
	 * 打开网页浏览器
	 */
	app.openBrowser = function(url){
		App.openWindow({
			url:'_www/html/browser.html',
			id:'browser',
			extras:{
				url:url
			}
		},true);
	}
	

	/**
	 * 第三方登录
	 * @param {String} type 登录平台 qq weixin sinaweibo
	 * @param {Function} Fun 登录成功后的回调函数 参数 {info,outLogin}
	 */
	app.authLogin = function(type, Fun) {
		var auths = null;

		var callback = function(ser) {
			Fun && Fun({
				info: ser.userInfo,
				outLogin: function() {
					authLogout(ser);
				}
			});
		}

		plus.nativeUI.showWaiting();

		plus.oauth.getServices(function(services) {
			auths = services;
			var ts = getServer(type);
			login(ts);
			setTimeout(function() {
				plus.nativeUI.closeWaiting();
			}, 800);
		}, function(e) {
			plus.nativeUI.closeWaiting();
			mui.toast("获取登录服务失败：" + e.message + " - " + e.code);
		});

		//获取登陆服务
		function getServer(type) {
			var len = auths.length;
			for(var i = 0; i < len; i++) {
				if(auths[i].id == type) {
					return auths[i];
				}
			}
			return null;
		}

		// 登录操作
		function login(s) {
			if(!s.authResult) {
				s.login(function(e) {
					mui.toast(s.description + " 登录认证成功！");
					authUserInfo(s);
				}, function(e) {
					mui.toast(s.description + " 登录认证失败！");
				});
			} else {
				mui.toast(s.description + " 已经登录认证！");
			}
		}

		// 登录认证信息
		function authUserInfo(s) {
			s.getUserInfo(function(e) {
				callback(s);
				// authLogout(s);
			}, function(e) {
				mui.toast("获取用户信息失败：" + e.message + " - " + e.code);
			});
		}

		//注销
		function authLogout(s) {
			if(s && s.authResult) {
				s.logout(function(e) {
					console.log(s.description + " 注销登录认证成功！");
				}, function(e) {
					console.log(s.description + " 注销登录认证失败！");
				});
			}
		}

	}

	/**
	 * 获取网络类型
	 */
	app.getNetwork = function() {
		var networkTypes = {};
		networkTypes[plus.networkinfo.CONNECTION_UNKNOW] = "unknow"; //未知
		networkTypes[plus.networkinfo.CONNECTION_NONE] = "none"; //未连接
		networkTypes[plus.networkinfo.CONNECTION_ETHERNET] = "line"; //有线网络
		networkTypes[plus.networkinfo.CONNECTION_WIFI] = "wifi"; //wifi
		networkTypes[plus.networkinfo.CONNECTION_CELL2G] = "2g"; //2g
		networkTypes[plus.networkinfo.CONNECTION_CELL3G] = "3g"; //3g
		networkTypes[plus.networkinfo.CONNECTION_CELL4G] = "4g"; //4g
		return networkTypes[plus.networkinfo.getCurrentType()];
	}

	/**
	 * 下载文件
	 * @param {String} url 文件地址
	 */
	app.downFile = function(url, callback) {

		var dtask = plus.downloader.createDownload(url, {
			method: "GET"
		}, function(d, status) {
			if(status == 200) {
				console.log("Download wgtu success: " + d.filename);
				callback && callback(d.filename);
			} else {
				callback && callback();
			}
		});
		dtask.addEventListener('statechanged', function(d, status) {
			console.log("statechanged: " + d.state);
		});

		dtask.start();
		return dtask;
		// 暂停下载任务 dtask.pause();
		// 取消下载任务 dtask.abort();
	}

	/**
	 * 打开wifi
	 */
	app.openWifi = function() {
		if($.os.ios) {
			plus.runtime.openURL("prefs:root=WIFI"); //ios需要手动打开
		} else if($.os.android) {
			var Context = plus.android.importClass("android.content.Context");
			var WifiManager = plus.android.importClass("android.net.wifi.WifiManager")
			var wifiManager = plus.android.runtimeMainActivity().getSystemService(Context.WIFI_SERVICE);
			wifiManager.setWifiEnabled(true); //false 为关闭
		}
	}

	/**
	 * Input获取焦点 弹出软键盘
	 * @param {HTMLInputElement} inputElem 
	 */
	app.showSoftInput = function(inputElem) {
		if(!inputElem) {
			return;
		}
		if(!inputElem.id) {
			this.input.id = inputElem.id = 'InputDefultId';
		}

		var nativeWebview, imm, InputMethodManager;
		var initNativeObjects = function() {
			if($.os.android) {
				var main = plus.android.runtimeMainActivity();
				var Context = plus.android.importClass("android.content.Context");
				InputMethodManager = plus.android.importClass("android.view.inputmethod.InputMethodManager");
				imm = main.getSystemService(Context.INPUT_METHOD_SERVICE);
				plus.android.importClass(nativeWebview);
				nativeWebview.requestFocusFromTouch();
				//强制显示软键盘 
				//imm.showSoftInput(nativeWebview,InputMethodManager.SHOW_FORCED);
				imm.toggleSoftInput(0, InputMethodManager.SHOW_FORCED);

			} else {
				nativeWebview.plusCallMethod({
					"setKeyboardDisplayRequiresUserAction": false
				});
			}
		};

		$.plusReady(function() {
			if(document.activeElement.id === inputElem.id) {
				return;
			}
			nativeWebview = plus.webview.currentWebview().nativeInstanceObject();
			initNativeObjects();
			setTimeout(function() {
				inputElem.focus();
			}, 200);
		});

	}

	//资源升级
	app.updateWgt = function(url) {
		var hz = url.split('.').pop();
		if(hz.indexOf('wgt') < 0 || hz.indexOf('apk')) {
			console.log('不是资源包');
			return;
		}

		//更新应用资源 
		var installWgt = function(path, callback) {
				plus.runtime.install(path, {}, function() {
					callback && callback(true);
				}, function(e) {
					callback && callback(false, e);
				});
			},
			w = plus.nativeUI.showWaiting("获取资源...");

		var task = this.downFile(url, function(path) {
			w.setTitle('安装资源中...')
			installWgt(path, function(state, e) {
				w.close();
				if(!state) {
					alert("安装wgt文件失败[" + e.code + "]：" + e.message);
					$.toast('更新失败稍后再试...');
					return;
				}
				console.log("安装wgt文件成功！");
				plus.nativeUI.alert("应用资源更新完成！", function() {
					plus.runtime.restart();
				});
			});
		});

	}

	//开始原生动画
	function startAnimation(type, bitmapT, bitmapB, callback) {
		var bm = bitmapB ? {
				bitmap: bitmapB
			} : {},
			tm = bitmapT ? {
				bitmap: bitmapT
			} : {};
		plus.nativeObj.View.startAnimation({
			type: type || 'pop-in' //默认
		}, bm, tm, function() {
			callback && callback(plus.nativeObj.View.clearAnimation);
		});
	}

	//webview截图
	function drawWebView(webview, bitmap, callback) {
		bitmap = bitmap || new plus.nativeObj.Bitmap('defultBitMap');
		webview.draw(bitmap, function() {
			callback && callback(bitmap);
		}, function(err) {
			callback && callback();
			console.log('截图错误：' + JSON.stringify(err))
		});
	}
	var bitmap, //动画上层截图
		bitmap2; //动画下层截图
	app.openWindow = function(op, when) {
		if(op.id && plus.webview.getWebviewById(op.id)) {
			plus.webview.close(op.id);
		}
		
		var nw = $.preload(op);
		bitmap =  bitmap || new plus.nativeObj.Bitmap('nwbitmap'+op.id); 
		
		//webview onloaded事件 
		when || plus.nativeUI.showWaiting('点我重开',{
			padlock:true
		}); //'',{color:'#636ee1',loading:{display:'inline'}});
		nw[when ? 'onloading' : 'onloaded'] = function(e) {
				when || plus.nativeUI.closeWaiting();
				//开始截图open
				drawWebView(nw, bitmap, function() {});
				bitmap2 = bitmap2 || new plus.nativeObj.Bitmap('cwbitmap'+op.id);
				drawWebView(cw.parent() || cw, bitmap2, function() {});
				
				startAnimation('pop-in', bitmap,bitmap2, function(close) { //bitmap是新页面截图，bitmap2是当前页面截图，这样的动画效果完美
					plus.webview.show(nw, 'none',0, function() {
						close()
					});
					
					
					nw.addEventListener('hide', function() {
						
						drawWebView(nw, bitmap, function() {}); //在这里做了更新 截图
						startAnimation('pop-out',bitmap, bitmap2, function(close) { //这里不完美，但却暂时解决不了。1：bitmap新页面的截图没有更新，2：hide时再播放动画
							//关闭页面原生动画
							close();
						});
						nw.close();
					});
				});

			}
			//新页面在load的同时，截图当前页面
			var cw = plus.webview.currentWebview();
			//drawWebView(cw.parent() || cw, bitmap2, function() {});
			
		return nw;
	}

	/**
	 * 炒鸡无敌跨webview显示在最顶端的真男人alert
	 * 
	 */
	app.superAlert = function(title, text, btnArr, callback,maskCallback) {
		var vInfo = {
			rect: {
				top: '30%',
				left: '10%',
				width: '80%',
				height: '40%'
			},
			title: {
				dst: {
					top: '5%',
					width: '80%',
					left: '10%',
					height: '20%'
				},
				styles: {
					color: '#666',
					size: '18px'
				}
			},
			ctent: {
				dst: {
					top: '25%',
					width: '90%',
					left: '5%',
					height: '50%'
				},
				src: {},
				styles:{
					color: '#333',
					size: '20px'
				}
			}
		};
		var view = plus.nativeObj.View.getViewById('superView'),
			maskView = plus.nativeObj.View.getViewById('superMaskView'),
			bitmap = plus.nativeObj.Bitmap.getBitmapById('superBitmap');
		view && view.clear();
		maskView && maskView.clear();
		bitmap && bitmap.clear();
		view = new plus.nativeObj.View('superView', vInfo.rect);
		maskView = new plus.nativeObj.View('superMaskView');
		//bitmap = new plus.nativeObj.Bitmap('superBitmap');

		//绘制遮罩
		maskView.drawRect("rgba(0,0,0,0.5)");
		//监听遮罩 防止透传事件
		maskView.addEventListener('click', function() {
			maskCallback && maskCallback();
		});

		//绘制弹窗
		view.drawRect("#ddd");
		//绘制弹窗标题
		view.drawText(title, vInfo.title.dst, vInfo.title.styles);

		//如果是图片
		if((function(hz, arr) {
				for(var i = 0, l = arr.length - 1; i < l; i++) {
					if(arr[i] == hz) {
						return !0;
					}
				}
			}(text.split('.').pop(), [
				'jpg', 'png', 'jpeg'
			]))) {
			text = plus.io.convertLocalFileSystemURL(text);
			bitmap = new plus.nativeObj.Bitmap('superBitmap');
			bitmap.load(text, function() {
				view.drawBitmap(bitmap,vInfo.ctent.src,vInfo.ctent.dst);
			}, function() {});
		} else {
			//绘制弹窗内容
			view.drawText(text, vInfo.ctent.dst,vInfo.ctent.styles);
		}
		
		//转化为数组
		if(!Array.isArray(btnArr)) {
			btnArr = [btnArr];
		}
		
		//百分比转小数
		function toPoint(percent){
		    var str=percent.replace("%","");
		    str= str/100;
		    return str;
		}
		
		var btnInfo = {
			w : (100 / btnArr.length).toFixed(0),//每个btn宽度
			sw : plus.screen.resolutionWidth//屏幕宽度
		};
		btnInfo.offl = toPoint(vInfo.rect.left)*btnInfo.sw;//左偏移 
		btnInfo.offw = toPoint(btnInfo.w)*(btnInfo.sw*toPoint(vInfo.rect.width));
		btnArr.map(function(item, index) {
			view.drawText(item, {
				top: '80%',
				left: btnInfo.w * index + '%',
				width: btnInfo.w + '%',
				height: '10%'
			}, {
				color: '#438', 
				size: '20px'
			});
		});

		//设置监听区域
		view.setTouchEventRect({
			top: '70%',
			width: '100%',
			height: '20%'
		});
		var oback = $.back;
		$.back = function(){};
		//监听弹窗
		view.addEventListener('click', function(e) {
			maskView.clear();
			view.clear();
			$.back = oback;
			//计算click下标的位置
			var index = Math.floor((e.pageX-btnInfo.offl) / btnInfo.offw);
			callback && callback(index);
		});

		maskView.show();
		view.show();
		 
	}

	return app;
}(mui));